from .fpo_patent import *
from .google_patent import *
from .utils import createDirs,log
from .google_patent_async import *
from .google_patent_common import *
__all__ = [
    'downloadFile',
    'createDirs',
    'getFpoPatentInfo',
    "getFpoPatentInfoByUrl",
    "getFpoSearchResult",
    "downloadFpoPdf",
    "downloadFpoPdfByUrl",
    "autoFpoSpider",
    "getGooglePatentInfo",
    "getGooglePatentInfoByUrl",
    "getGooglePatentInfoAsync",
    "getGooglePatentInfoByUrlAsync",
    "downloadGooglePdfAsync",
    "getHtmlAsync",
    "parseGooglePatentInfoHtml"
]